import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.util.Queue;
import java.util.LinkedList;

/**
 * The abstract class GameView is a template to make game views using SimpleGame
 * g2d game library.
 *
 * @author Jason Petty
 * @version 1.0
 */
public abstract class GameView extends SimpleGame
{
    // instance variables
    private JFrame gameFrame;
    private JPanel gamePanel;
    
    public static final int FRAME_WIDTH = 500;
    public static final int FRAME_HEIGHT = 700;
    public static final int SCREEN_BOTTOM = 590;
    public static final int SCREEN_TOP = 70;
    public static final int SCREEN_LEFT = 20;
    public static final int SCREEN_RIGHT = 460;
    public static final Color BACKGROUND_COLOR = Color.black;
    
    public Queue<SimpleGame> drawQueue;
    public int score;
    
    public GameView()
    {
        // initialise instance variables
        gameFrame = new JFrame();
        gameFrame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        gameFrame.setSize(FRAME_WIDTH, FRAME_HEIGHT);
        gameFrame.setResizable(false);
        
        this.gamePanel = new GamePanel();
        gameFrame.getContentPane().add(BorderLayout.CENTER, gamePanel);
        
        drawQueue = new LinkedList<SimpleGame>();
        score = 0;
    }
    
    public void showGame(boolean value)
    {
        if ( value == true )
        {
            gameFrame = centerFrame(gameFrame);
        }
        gameFrame.setVisible(true);
    }
    
    private JFrame centerFrame(JFrame frame)
    {
        Dimension screenInfo = Toolkit.getDefaultToolkit().getScreenSize();
        frame.setLocation(screenInfo.width/2-frame.getSize().width/2, screenInfo.height/2-frame.getSize().height/2);
        return frame;
    }
    
    public void repaint()
    {
        gamePanel.repaint();
    }
    
    public void addScore(int value)
    {
        score = score + value;
    }
    
    public void resetScore()
    {
        score = 0;
    }
    
    public int getScore()
    {
        return score;
    }
    
    public void setGamePanel(JPanel panel)
    {
        this.gamePanel = panel;
        gameFrame.getContentPane().add(BorderLayout.CENTER, gamePanel);
    }
    
    public void setGamePanel(JPanel panel, KeyListener keyListener)
    {
        this.gamePanel = panel;
        gameFrame.getContentPane().add(BorderLayout.CENTER, gamePanel);
        gamePanel.setFocusable(true);
        gamePanel.addKeyListener(keyListener);
    }
    
    public void addKeyListener(KeyListener keyListener)
    {
        gamePanel.setFocusable(true);
        gamePanel.addKeyListener(keyListener);
    }
    
    public void fillQueue(LinkedList<SimpleGame> drawList)
    {
        // Using this queue require polymorphism.
        drawQueue = new LinkedList<SimpleGame>();
        for ( int count = 0; count < drawList.size(); count++ )
        {
            drawQueue.add(drawList.get(count));
        }        
    }
    
    class GamePanel extends JPanel
    {
        // This paint Component is meant to be overridden to add your specific game code.
        public void paintComponent(Graphics g)
        {
            Graphics2D g2d = (Graphics2D)g;
            g2d.setBackground(BACKGROUND_COLOR);
            g2d.clearRect(0,0,this.getWidth(), this.getHeight());
                       
            while ( !drawQueue.isEmpty() )
            {
                // Put Draw Code Here by casting the simpleGame object to the appropriate object and
                // drawing the objects out of the cue.
            }
            
            // This area is used to draw anything else you want to draw that is not queued up.
        }
    }
    
}
