import java.awt.event.*;
import javax.swing.*;

/**
 * The abstract class GameController is a template to create a controller using
 * SimpleGame g2d game library.
 *
 * @author Jason Petty
 * @version 1.0
 */
public abstract class GameController extends SimpleGame implements ActionListener
{
    // instance variables
    private GameView gameView;
    private Timer timer;
    public static final int DELAY = 25;
    
    public void setGameView(GameView gameView)
    {
        this.gameView = gameView;
    }
    
    public void setGameView(GameView gameView, boolean initGameView,KeyListener keyListener, GameController gameController)
    {
        this.gameView = gameView;
        if  ( initGameView == true )
        {
            getGameView().showGame(initGameView);
        }
        
        if ( keyListener != null )
        {
            getGameView().addKeyListener(keyListener);
        }
        
        if ( gameController != null )
        {
            setTimer(gameController);
            startTimer();
        }       
    }
    
    public GameView getGameView()
    {
        return this.gameView;
    }
    
    public void setTimer(ActionListener actionListener)
    {
        if ( timer == null )
        {
            timer = new Timer(DELAY, actionListener);
        }
    }
    
    public void setTimer(ActionListener actionListener, int delay)
    {
        if ( timer == null )
        {
            timer = new Timer(delay, actionListener);
        }
    }
    
    public void setTimerDelay(int delayValue)
    {
        if ( timer.isRunning() == true && timer != null )
        {
            timer.stop();
            timer.setDelay(delayValue);
            timer.start();
        }
    }
    
    public void startTimer()
    {
        timer.start();
    }
    
    public void stopTimer()
    {
        if ( timer.isRunning() && timer != null )
        {
            timer.stop();
        }
    }
    
    public void actionPerformed(ActionEvent e) {
            getGameView().repaint();
    }
        
    class GameKeyListener implements KeyListener
    {
        public void keyTyped(KeyEvent e) {
            // This inner class is meant to be overriden with specific game code.
            
        }
        public void keyPressed(KeyEvent e) {
            // This inner class is meant to be overriden with specific game code.                
        }
        public void keyReleased(KeyEvent e) {
            // This inner class is meant to be overriden with specific game code.
        }
    }
}
