import java.awt.event.*;
import javax.swing.*;
import java.util.LinkedList;
import java.awt.*;
import java.util.Random;

/**
 * The class SnakeController is used to control the entire game.
 *
 * @author Jason Petty
 * @version 1.0
 */
public class SnakeController extends GameController
{
    // instance variables
    private static int INITIAL_SNAKE_SIZE = 5;
    private static int STARTING_X = 180;
    private static int STARTING_Y = 200;
    private static Color SNAKE_COLOR = Color.yellow;
    private static int SCREEN_NORTH = 90;
    private static int SCREEN_SOUTH = 570;
    private static int SCREEN_WEST = 40;
    private static int SCREEN_EAST = 420;
    
    private static int SNAKE_DELAY_MAX = 125;
    private static int SNAKE_DELAY_MIN = 25;
    
    private int currentDelay;
    private boolean changeDirection;
    private boolean addSegment;
    private boolean immunityActivated;
    private int immunityCounter;
    
    private String direction;
    private Random random;
    
    private int segmentCounter;
    
    private boolean gameOver;
    
    // Required to use the shape drawing queue
    
    // This one holds the snake
    private LinkedList<SimpleGame> spriteList;
    // This one holds fruit
    private LinkedList<SimpleGame> fruitList;

    /**
     * Constructor for objects of class SnakeController
     */
    public SnakeController()
    {
        // initialise instance variables
        changeDirection = false;
        addSegment = false;
        currentDelay = SNAKE_DELAY_MAX;
        segmentCounter = 0;
        
        gameOver = false;
        immunityActivated = false;
        immunityCounter = 0;
        
        
        direction = "east";
        random = new Random();
        
        setGameView(new SnakeView(), true,new GameKeyListener(), null);
        setTimer(this, currentDelay);
        startTimer();
        
        ((SnakeView)getGameView()).setSpeed("Slow");
        
        // Required to use the shape drawing queue
        spriteList = new LinkedList<SimpleGame>();
        fruitList = new LinkedList<SimpleGame>();
        
        // Create a snake
        int x = STARTING_X;
        int y = STARTING_Y;
        for ( int counter = 0; counter < INITIAL_SNAKE_SIZE; counter++ )
        {
            SnakeSegment segment = new SnakeSegment(x, y, SNAKE_COLOR);
            segmentCounter++;
            segment.setId(segmentCounter);
            spriteList.add(segment);
            x = x+10;
        }
        ((SnakeView)getGameView()).setSegments(INITIAL_SNAKE_SIZE);
        
        createApple();        
    }
    
    private void restartGame()
    {
        changeDirection = false;
        addSegment = false;
        direction = "east";
        getGameView().resetScore();
        ((SnakeView)getGameView()).resetSegments();
        ((SnakeView)getGameView()).resetGameOver();
        
        segmentCounter = 0;
        
        gameOver = false;
        immunityActivated = false;
        immunityCounter = 0;
        
        currentDelay = SNAKE_DELAY_MAX;
        setTimerDelay(currentDelay);
        
        ((SnakeView)getGameView()).setSpeed("Slow");
        
        // Required to use the shape drawing queue
        spriteList = new LinkedList<SimpleGame>();
        fruitList = new LinkedList<SimpleGame>();
        
        // Create a snake
        int x = STARTING_X;
        int y = STARTING_Y;
        for ( int counter = 0; counter < INITIAL_SNAKE_SIZE; counter++ )
        {
            SnakeSegment segment = new SnakeSegment(x, y, SNAKE_COLOR);
            segmentCounter++;
            segment.setId(segmentCounter);
            spriteList.add(segment);
            x = x+10;
        }
        ((SnakeView)getGameView()).setSegments(INITIAL_SNAKE_SIZE);
        
        createApple();       
    }
    
    
    private void createApple()
    {
        int appleX = random.nextInt((SCREEN_EAST-SCREEN_WEST))+SCREEN_WEST;
        int appleY = random.nextInt((SCREEN_SOUTH-SCREEN_NORTH))+SCREEN_NORTH;
        
        Apple apple = new Apple(appleX, appleY, Color.red);
        fruitList.add(apple);
    }
    
    private void moveApple(Sprite sprite)
    {
        int appleX = random.nextInt((SCREEN_EAST-SCREEN_WEST))+SCREEN_WEST;
        int appleY = random.nextInt((SCREEN_SOUTH-SCREEN_NORTH))+SCREEN_NORTH;
        sprite.updateSprite(appleX, appleY);
    }
    
    private void speedUpGame()
    {
        currentDelay = currentDelay - 3;
        if (currentDelay <  SNAKE_DELAY_MIN)
            currentDelay = SNAKE_DELAY_MIN;
        
        String speedText = "Slow";
        if ( currentDelay == SNAKE_DELAY_MIN )
        {
            speedText = "Turbo";
        }
        else if ( currentDelay < 50 )
        {
            speedText = "Smokin";
        }
        else if ( currentDelay < 100 )
        {
            speedText = "Cruise";
        }
        
        setTimerDelay(currentDelay);  
        ((SnakeView)getGameView()).setSpeed(speedText);
    }
    
    public void updateSpriteList()
    {         
        // Moves the snake
        SnakeSegment headSprite = (SnakeSegment)spriteList.getLast();
        SnakeSegment sprite = (SnakeSegment)spriteList.removeFirst();        
        sprite.moveSprite(direction, 0, headSprite.getX(), headSprite.getY());
        spriteList.addLast(sprite);
        
        // Checks for fruit swallowed
        for ( int anotherCounter = 0; anotherCounter < fruitList.size(); anotherCounter++ )
        {
            Sprite target = (Sprite)fruitList.get(anotherCounter);
            if ( target instanceof Apple )
            {
                boolean results = sprite.getShape().intersects(target.getShape().getBounds2D());
                if ( results == true )
                {
                    // Hit
                    addSegment = true;
                    immunityActivated = true;
                    immunityCounter = 0;
                    
                    moveApple(target);
                    getGameView().addScore(5);
                    growSnake();
                    speedUpGame();
                    addSegment = false;
                }

            }
            
            // checks to see if snake hits itself
            if ( immunityActivated == false )
                checkSnake();
        }
                       
    }
    
    private void checkSnake()
    {
        for ( int counter = 0; counter < spriteList.size(); counter++ )
        {
            Sprite sprite = (Sprite)spriteList.get(counter);
            for ( int anotherCounter = 0; anotherCounter < spriteList.size(); anotherCounter++ )
            {
                Sprite target = (Sprite)spriteList.get(anotherCounter);
                if ( target.getId() != sprite.getId() )
                {
                    boolean results = sprite.getShape().intersects(target.getShape().getBounds2D());
                    if ( results == true )
                    {
                        // Snake hit itself
                        gameOver = true;
                        ((SnakeView)getGameView()).setGameOver();
                    }
                }
            }
        }
        
    }
    
    private void immunityCheck()
    {
        if ( immunityActivated == true )
        {
            immunityCounter++;
            if ( immunityCounter > (spriteList.size()+1) )
            {
                immunityActivated = false;
                immunityCounter = 0;
            }
        }
    }
    
    private void growSnake()
    {
        Sprite sprite = (Sprite)spriteList.getLast();
        int x = sprite.getX();
        int y = sprite.getY();
        
        if ( direction == "north" )
        {
            y = y+10;
        }
        else if ( direction == "south" )
        {
            y = y-10;
        }
        else if ( direction == "west" )
        {
            x = x + 10;
        }
        else
        {
            x = x -10;
        }
       
        SnakeSegment newSegment = new SnakeSegment(x, y, SNAKE_COLOR);
        segmentCounter++;
        newSegment.setId(segmentCounter);
        spriteList.addLast(newSegment);  
        ((SnakeView)getGameView()).setSegments(spriteList.size());
    }
    
    private LinkedList combineLists()
    {
        LinkedList combinedList = new LinkedList<SimpleGame>();
        for ( int counter = 0; counter < spriteList.size(); counter++ )
        {
            combinedList.add(spriteList.get(counter));
        }
        for ( int counter = 0; counter < fruitList.size(); counter++)
        {
            combinedList.add(fruitList.get(counter));
        }
        return combinedList;
    }
    
    public void actionPerformed(ActionEvent e) {
        immunityCheck();
        if ( gameOver == false )
        {
            updateSpriteList();
            getGameView().fillQueue(combineLists());
        }
        getGameView().repaint();   
    }
    
    class GameKeyListener implements KeyListener
    {
        public void keyTyped(KeyEvent e) {
            // This inner class is meant to be overriden with specific game code.

        }
        public void keyPressed(KeyEvent e) {
            // This inner class is meant to be overriden with specific game code.                

        }
        public void keyReleased(KeyEvent e) {
            // This inner class is meant to be overriden with specific game code.
            switch(e.getKeyCode()){
                case KeyEvent.VK_DOWN:  if ( direction != "north" && addSegment == false && gameOver == false)
                                            direction = "south";
                                        break;
                case KeyEvent.VK_LEFT:  if ( direction != "east" && addSegment == false && gameOver == false)
                                            direction = "west";
                                        break;
                case KeyEvent.VK_RIGHT: if ( direction != "west" && addSegment == false && gameOver == false)
                                            direction = "east";
                                        break; 
                case KeyEvent.VK_UP:    if ( direction != "south" && addSegment == false && gameOver == false )
                                            direction = "north";
                                        break;    
                case KeyEvent.VK_R:     restartGame();
                                        break;      
                default:
            }
        }
    }
}
